import java.io.*;
import DE.fub.inf.JVM.JavaClass.*;
import DE.fub.inf.JVM.Constants;
import DE.fub.inf.JVM.ClassPath;

/**
 * Read class file(s) and display its contents.
 *
 * @version $Id: listclass.java,v 1.7 1998/10/16 12:48:09 dahm Exp $
 * @author  <A HREF="http://www.inf.fu-berlin.de/~dahm">M. Dahm</A>
 */
public class listclass implements Constants {
  public static void main(String argv[])
  { 
    String[]    file_name = new String[argv.length];
    int         files=0;
    ClassParser parser=null;
    JavaClass   java_class;
    boolean     code=false, constants=false;
    String      zip_file=null;

    try {
      ClassPath class_path = new ClassPath();

      /* Parse command line arguments.
       */
      for(int i=0; i < argv.length; i++) {
	if(argv[i].charAt(0) == '-') {  // command line switch
	  if(argv[i].equals("-constants"))
	    constants=true;
	  else if(argv[i].equals("-code"))
	    code=true;
	  else if(argv[i].equals("-zip"))
	    zip_file = argv[++i];
	}
	else { // add file name to list
	  file_name[files++] = argv[i];
	}
      }
	
      if(files == 0) {
	System.err.println("list: No input files specified");
      }
      else {
	for(int i=0; i < files; i++) {
	  if(file_name[i].endsWith(".class")) {
	    if(zip_file == null)
	      parser = new ClassParser(file_name[i]);
	    else
	      parser = new ClassParser(zip_file, file_name[i]);
	  }
	  else {
	    InputStream is   = class_path.getInputStream(file_name[i]);
	    String      name;

	    if(is != null)
	      name = file_name[i].replace('.', File.separatorChar) + ".class";
	    else {
	      name = file_name[i];
	      is   = new FileInputStream(name);
	    }
	      
	    parser = new ClassParser(is, name);
	  }

	  java_class = parser.parse();                // Initiate the parsing
	  System.out.println(java_class);             // Dump the contents

	  if(constants) // Dump the constant pool ?
	    System.out.println(java_class.getConstantPool());

	  if(code) // Dump the method code ?
	    printCode(java_class.getMethods());
	}
      }	  
    } catch(Exception e) {
      e.printStackTrace();
    }
  }        
  /**
   * Dump the disassembled code of all methods in the class.
   */
  public static void printCode(DE.fub.inf.JVM.JavaClass.Method[] methods) {
    for(int i=0; i < methods.length; i++) {
      System.out.println(methods[i]); // public static void main(String argv[])

      Code code = methods[i].getCode();
      if(code != null)
	System.out.println(code);
    }
  }
}
